#! perl -w

#  Licensed Materials - Property of IBM                               
#  Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
#  Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
#  U.S. Government Users Restricted Rights -  Use, duplication or     
#  disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
#  

# This script performs two functions: It translates object file lists
# (identified by %file on the command line) to linker command files,
# and adds the constructor/destructor tables to the executable.
#
# Example usage:
# ===============
# perl link.pl ARCH=386 -o TopCapsule \
#   main.o %OLIST user_obj.o -LX -lx ...
#
# The general algorithm:
#   Translate each OLIST to OLIST.cmd
#   ld386 -r -o RTmunch.o main.o -T OLIST.cmd user_obj.o -LX -lx
#   nm386 RTmunch.o | munch > RTctor.c
#   cc386 -traditional -o RTctor.o -c RTctor.c
#   ld386 -r -o TopCapsule RTctor.o RTmunch.o

# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

sub mySystem
{
    my( $OS, $ROSERT_HOST );

    $OS          = $ENV{'OS'}          || '?';
    $ROSERT_HOST = $ENV{'ROSERT_HOST'} || '?';

	return system( join( ' ', @_ ) );
}

sub report_ges_error
{
    my( $ERROR ) = @_;

    print "description '$ERROR' severity 'error'\n";
    print "result 'failure'\n";

    exit 1;
}

sub show_usage
{
    &report_ges_error( "Usage: $0 ARCH=<arch> options ... " );
}

&show_usage() unless $#ARGV >= 1;

if( $ARGV[0] =~ m{^ARCH=(\w+)$} )
{
    $arch = $1;
    shift @ARGV;
}
else
{
    &show_usage();
}

&report_ges_error( "WIND_BASE is not set" )
	unless( $WIND_BASE = $ENV{'WIND_BASE'} );

#&report_ges_error( "WIND_HOST_TYPE is not set" )
#	unless( $WIND_HOST_TYPE = $ENV{'WIND_HOST_TYPE'} );

# commands
$cc_cmd  = "cc$arch";
$ld_cmd  = "ld$arch";

if ( $arch eq 'pentium' )
{
$munch   = "tclsh \"$WIND_BASE/host/resource/hutils/tcl/munch.tcl\""
		 . " -asm $arch";
}
else
{
$munch   = "tclsh \"$WIND_BASE/host/resource/hutils/tcl/munch.tcl\""
		 . " -c $arch";
}

$nm_cmd  = "nm$arch --extern-only --no-sort";

# filenames
$prelink = 'RTmunch.o';
$ctor_c  = 'RTctor.c';
$ctor_o  = 'RTctor.o';
$output  = 'a.out';

$status   = 0;                              # assume all is well
$cmdfile  = 'ld000';                        # command filename pattern
@tmpfiles = ( $prelink, $ctor_c, $ctor_o ); # temporary files

# Translate all indirect object file lists (identified by %) into linker
# command files for use in the prelink step.

@command  = ( $ld_cmd, '-r', '-o', $prelink );

{
	@script_intro = ();
}

foreach $arg ( @ARGV )
{
    if( $arg eq '-o' )
    {
	$lasto = 1;
    }
    elsif( $lasto )
    {
	$output = $arg;
	$lasto  = 0;
    }
    elsif( $arg =~ m{^%(.*)$} )
    {
	$in = $1;
	if( ! open( IN, "<$in" ) )
	{
	    print STDERR "link.pl: Can't read '$in': $!\n";
	    $status = 1;
	}
	else
	{
	    $out = "$cmdfile.cmd";
	    ++$cmdfile;
	    if( ! open( OUT, ">$out" ) )
	    {
		print STDERR "link.pl: Can't create '$out': $!\n";
		$status = 1;
	    }
	    else
	    {
		while( <IN> )
		{
		    chomp;
		    print OUT "INPUT(", $_, ".o)\n";
		}
		close( OUT );

		push @command, @script_intro, $out;
		@script_intro = ();
		push @tmpfiles, $out;
	    }
	    close( IN );
	}
    }
    else
    {
		push @command, $arg;
    }
}

# prelink everything
$status = &mySystem( @command ) if $status == 0;

# munch the big object file
$status = &mySystem( "$nm_cmd $prelink | $munch > $ctor_c" )
				if $status == 0;

# compile the constructor/destructor tables
if ( $arch eq 'pentium' )
{
$status = &mySystem( $cc_cmd, '-mcpu=i486 -fno-builtin -MD -MP', '-fdollars-in-identifiers', '-ansi',
				 '-DCPU=SIMNT', '-DTOOL_FAMILY=gnu', '-DTOOL=gnu',
				 '-DUSE_THREADS', '-o', $ctor_o, '-c', $ctor_c )
					if $status == 0;
}
else
{
$status = &mySystem( $cc_cmd, '-mcpu=603 -fno-builtin -MD -MP', '-fdollars-in-identifiers', '-ansi',
				 '-DCPU=PPC603', '-DTOOL_FAMILY=gnu', '-DTOOL=gnu -D_WRS_KERNEL',
				 '-DUSE_THREADS', '-o', $ctor_o, '-c', $ctor_c )
					if $status == 0;
}

# final link of the application
$filename = $WIND_BASE . "/target/h/tool/gnu/ldscripts/link.OUT" ;
$status = &mySystem( $cc_cmd,  '-r -nostdlib -Wl,-X -T ', $filename, '-o', $output, $ctor_o, $prelink )
				if $status == 0;

# remove our temporary files
unlink( @tmpfiles );

exit $status if $status < 256;
exit $status / 256;
